@@ -905,7 +905,7 @@ self.mixers = [{', '.join([call(x) for x in json_ai.model["args"]["submodels"]])
 trained_mixers = []
 for mixer in self.mixers:
     try:
-        mixer.fit(encoded_train_data, encoded_dev_data)
+        self.fit_mixer(mixer, encoded_train_data, encoded_dev_data)
         trained_mixers.append(mixer)
     except Exception as e:
         log.warning(f'Exception: {{e}} when training mixer: {{mixer}}')
@@ -1115,6 +1115,10 @@ class Predictor(PredictorInterface):
         # Fit predictors to estimate target
 {fit_body}
 
+    @timed
+    def fit_mixer(self, mixer, encoded_train_data, encoded_dev_data) -> None:
+        mixer.fit(encoded_train_data, encoded_dev_data)
+
     @timed
     def analyze_ensemble(self, enc_data: Dict[str, pd.DataFrame]) -> None:
         # Evaluate quality of fit for the ensemble of mixers
